# auto generated by update_py.py

import threading
import time

import tlclient.trader.message_common as message_common
import tlclient.trader.message_market as message_market
import tlclient.trader.message_trade as message_trade
from tlclient.trader.constant import ExchangeID, OffsetFlag, OrderType, MsgType
from tlclient.linker.constant import NotificationType
from tlclient.linker.fist import Fist
from tlclient.linker.frame import Frame
from tlclient.linker.utility import bytify
from tlclient.linker.timer import Timer


class Gateway(Fist):

    def __init__(self, acc_name, gateway_name, env_name, addr, fist_type):
        Fist.__init__(self, acc_name, fist_type, env_name)
        self.set_master_addr(addr)
        self.create_fist()
        self.router_name = None
        self.gateway_name = gateway_name
        # init heat beat
        self.hb = message_common.GatewayHeartBeat()
        self.hb.fist_type = self.fist_type
        self.hb.fist_name = bytify(self.fist_name)
        self.hb.gateway_name = bytify(self.gateway_name)
        self.hb.is_valid = False
        # init mutex
        self.push_mutex = threading.Lock()

    def init(self, router_name):
        self.reg_req(router_name)
        self.reg_sub(router_name)
        self.reg_push(router_name)
        self.router_name = router_name

    def safe_push(self, obj, msg_type, req_id=0):
        self.push_mutex.acquire()
        self.push(self.router_name, obj, msg_type, req_id)
        self.push_mutex.release()

    def start(self):
        Fist.start(self)
        t = threading.Thread(target=self.heart_beat)
        t.setDaemon(True)
        t.start()

    def heart_beat(self, sec_interval=5):
        while True:
            self.safe_push(self.hb, MsgType.GTW_HEART_BEAT)
            time.sleep(sec_interval)

    # --------- utilities for gateway ---------
    #
    def push_connection_status_change(self, connected, err_id=0, err_msg=''):
        # init conn_status
        conn_status = message_common.GatewayConnectionStatus()
        conn_status.fist_type = self.fist_type
        conn_status.fist_name = bytify(self.fist_name)
        conn_status.gateway_name = bytify(self.gateway_name)
        conn_status.connected = connected
        conn_status.err_id = err_id
        conn_status.err_msg = bytify(err_msg)

        self.hb.is_valid = connected
        self.safe_push(conn_status, MsgType.GTW_CONNECTION)
        title = self.fist_name
        content = self.fist_name + " connected"
        if not connected:
            content = self.fist_name + " disconnected"
        notificaton_type = NotificationType.SYSTEM
        self.notify(title, content, notificaton_type)
        self.logger.info('conn_status: ' + str(conn_status))
