# auto generated by update_py.py
import argparse
import json
import multiprocessing
import tempfile
import time
import os
import subprocess
import sys
import traceback

from linker import __version__ as linker_version

from . import __version__ as trader_version
from .gun_helpers import (CONFIG_DEFAULT_PATH, CONFIG_FOLDER, FIST_FAIL_MARK,
                          FIST_READY_MARK)

LOGO_PRINT = '''
  _                 _                 _ _       _
 | |_ _ __ __ _  __| | ___ _ __ ___  | (_)_ __ | | __
 | __| '__/ _` |/ _` |/ _ \ '__/ __| | | | '_ \| |/ /
 | |_| | | (_| | (_| |  __/ |  \__ \_| | | | | |   <
  \__|_|  \__,_|\__,_|\___|_|  |___(_)_|_|_| |_|_|\_\

    ----------------
    info@puyuan.tech
    trader: {}
    linker: {}
    ----------------
'''.format(trader_version, linker_version)


class ConsoleEntrance(object):

    FIST_TYPES = ['master', 'tg', 'mg', 'tr', 'mr', 'oms', 'rms', 'bs', 'algo', 'rr']  # ALL fist type supported

    HELP_MSG = '''
----------------------
gun <command> [<args>]

The most commonly used boxing commands are:
    help            show help info
    ctl             control panel for services
    version         get version
    config          config related ops
    status          show master status
    start           start process in background, foreground exit when inited
    stop            stop with a fist_name, 'gun stop \*' to stop all fists except master
    run             run process in foreground
    account         config accounts
    key             generate rsa keys
    db              db operations

    encrypt/decrypt to encrypt/decrypt data

    supported fist_type: {}
----------------------
'''.format(FIST_TYPES)

    def __init__(self):
        pass

    def parse_and_execute(self):
        entry_parser = argparse.ArgumentParser(description='Trader Command Tool', usage=self.HELP_MSG)
        entry_parser.add_argument('command', help='Subcommand to run')
        # parse_args defaults to [1:] for args, but you need to
        # exclude the rest of the args too, or validation will fail
        args = entry_parser.parse_args(sys.argv[1:2])
        if not hasattr(self, args.command):
            print('Unrecognized command: ' + args.command + '\n')
            entry_parser.print_help()
            exit(1)

        self.main_parser = argparse.ArgumentParser(description='gun run')
        self.main_parser.add_argument('-t', '--proc_type', choices=self.FIST_TYPES, help='process type')
        self.main_parser.add_argument('-f', '--fist_name', help='fist name')
        self.main_parser.add_argument('-g', '--gateway_name', help='gateway name')
        self.main_parser.add_argument('-a', '--acc_tag', help='account tag')
        self.main_parser.add_argument('-r', '--router_name', help='router name')
        self.main_parser.add_argument('-s', '--secondary_router_name', help='market router for mock')
        self.main_parser.add_argument('-p', '--package_name', help='package name')
        self.main_parser.add_argument('-d', '--db', action='store_true', help='enable db module')
        self.main_parser.add_argument('--dir', help='to specify directory')
        self.main_parser.add_argument('--ip', help='to specify ip(str)')
        self.main_parser.add_argument('--port', help='to specify port(number)')
        self.main_parser.add_argument('--proxy', action='store_true', help='use proxy settings in config.json')
        # use dispatch pattern to invoke method with same name
        getattr(self, args.command)()

    def help(self):
        print(self.HELP_MSG)

    def ctl(self):
        os.system("/usr/bin/supervisorctl -c {}/supervisor/supervisord.conf".format(CONFIG_FOLDER))

    def version(self):
        print(LOGO_PRINT)

    def status(self):
        os.system("linker_status")

    def stop(self):
        fist_name = sys.argv[2]
        # create a fist and call related function
        from trader.gun_helpers import GunClientHelper
        helper = GunClientHelper('__killer', CONFIG_DEFAULT_PATH)
        helper.stop(fist_name)

    def join(self):
        fist_name = sys.argv[2]
        from trader.gun_helpers import GunClientHelper
        helper = GunClientHelper('__joiner', CONFIG_DEFAULT_PATH)
        helper.join(fist_name, join_wait_seconds=2)

    def run(self):
        assert len(sys.argv) > 2, 'must specify -f --fist_type'
        fist_type = sys.argv[2]
        args = None
        if fist_type not in self.FIST_TYPES:
            if fist_type.startswith('-'):
                args = self.main_parser.parse_args(sys.argv[2:])
                fist_type = args.proc_type
        if fist_type not in self.FIST_TYPES:
            print(FIST_FAIL_MARK)
            self.main_parser.error('\n[UNSUPPORTED_TYPE] {} is not in supported list: {}'.format(fist_type, self.FIST_TYPES))
            return
        else:
            if args is None:
                args = self.main_parser.parse_args(sys.argv[3:])
            p = multiprocessing.Process(target=self._run_process, args=[fist_type, args])
            p.daemon = True
            p.start()
            try:
                p.join()
            except KeyboardInterrupt:
                pass

    def start(self):
        args = list(sys.argv)
        args[1] = 'run'
        cmd = ' '.join(args)
        # DO NOT let the temp file be deleted as the subprocess is still writing to it
        temp_file = tempfile.NamedTemporaryFile(delete=False)
        popen = subprocess.Popen(cmd, shell=True, stdout=temp_file.file, stderr=subprocess.STDOUT)
        with open(temp_file.name) as f:
            while True:
                time.sleep(0.1)
                lines = f.readlines()
                if len(lines):
                    for line in lines:
                        print(line, end='')
                        exit_code = 0 if FIST_READY_MARK in line else (1 if FIST_FAIL_MARK in line else -1)
                        if exit_code != -1:
                            exit(exit_code)

    def _run_process(self, fist_type, args):

        if fist_type in ['tg', 'mg']:
            if args.gateway_name is None or (fist_type == 'tg' and args.acc_tag is None):
                print(FIST_FAIL_MARK)
                self.main_parser.error('\n\n[MISSING_ARGS] tg requires -g -a, mg requires -g')
                return
            from trader.gun_helpers import Gateway, PythonGateway
            try:
                if args.package_name:
                    PythonGateway(args.package_name, fist_type, args.gateway_name, args.router_name, fist_name=args.fist_name, acc_tag=args.acc_tag, use_proxy=args.proxy).run()
                else:
                    Gateway(fist_type, args.gateway_name, args.acc_tag, args.router_name, args.secondary_router_name).run()
            except Exception as e:
                print(traceback.format_exc())
                print(FIST_FAIL_MARK)

        elif fist_type in ['tr', 'mr']:
            from trader.gun_helpers import Router
            Router(fist_type, args.fist_name).run()

        elif fist_type in ['oms']:
            from trader.gun_helpers import OrderService
            try:
                OrderService(args.fist_name, args.router_name).run()
            except:
                print(FIST_FAIL_MARK)

        elif fist_type in ['rms']:
            from trader.gun_helpers import RiskManagementService
            try:
                RiskManagementService(fist_name=args.fist_name, router1=args.router_name).run()
            except:
                print(FIST_FAIL_MARK)

        elif fist_type in ['bs']:
            from trader.gun_helpers import BasketServer
            try:
                BasketServer(fist_name=args.fist_name, trade_router=args.router_name).run()
            except:
                print(FIST_FAIL_MARK)

        elif fist_type in ['algo']:
            from trader.gun_helpers import AlgoServer
            try:
                if args.router_name:
                    trade_router, market_router = args.router_name.split('/')
                else:
                    trade_router, market_router = None, None
                AlgoServer(algo_type=args.fist_name, trade_router=trade_router, market_router=market_router).run()
            except Exception as e:
                print('[error] {} (traceback){}'.format(e, traceback.format_exc()))
                print(FIST_FAIL_MARK)

        elif fist_type in ['rr']:
            from trader.gun_helpers import GunRecorder
            router_names = args.router_name.split('/') if args.router_name else None
            try:
                _ip = args.ip or 'localhost'
                _port = args.port or os.environ['KDB_PORT']
                GunRecorder(args.fist_name, router_names, args.dir, _ip, _port).run()
            except:
                print(traceback.format_exc())
                print(FIST_FAIL_MARK)

        elif fist_type in ['master']:
            import liblinker
            master = liblinker.Master()
            master.start()
            print(FIST_READY_MARK)
            master.join()

        else:
            print(FIST_FAIL_MARK)
            raise NotImplementedError('process type "{}" not implemented yet'.format(fist_type))

    def config(self):
        from trader.gun_helpers import ConfigHelper

        parser = argparse.ArgumentParser(description="gun update_config")
        me_group = parser.add_mutually_exclusive_group(required=True)
        me_group.add_argument('-u', '--upgrade', action="store_true", help='to upgrade config file')
        me_group.add_argument('-c', '--check', action='store_true', help='to check config file')
        parser.add_argument('-d', '--config_dir', type=str, default=CONFIG_FOLDER, help='directory where config file was stored')
        args = parser.parse_args(sys.argv[2:])

        config_helper = ConfigHelper(args.config_dir)
        if args.upgrade:
            # backup config file
            config_helper.backup_config()
            # upgrade config file
            config_helper.update_config()
            print('current_version: {}'.format(config_helper.current_config_version))
            config_helper.dump_config()
            print("finished updating config")

        elif args.check:
            print('config path: {}'.format(config_helper.config_path))
            config_helper.check_version()

        else:
            raise NotImplementedError()

    def account(self):
        from .helpers import AccountHelper, KeyHelper

        parser = argparse.ArgumentParser(description="gun update_config")
        parser.add_argument('action', choices=['list', 'new', 'delete'], help='action to take')
        parser.add_argument('-a', '--acc_tag', help='account name')
        parser.add_argument('-k', '--key_name', help='encryption key')
        parser.add_argument('-c', '--content', nargs='*', default=[], help='config content, e.g. key1:value1 key2:value2')
        parser.add_argument('-f', '--force', action='store_true', help='force save/delete')
        args = parser.parse_args(sys.argv[2:])

        acc_helper = AccountHelper()
        if args.action == 'new':
            assert None not in [args.acc_tag], '-a is required'
            if args.key_name is not None:
                assert KeyHelper.key_exists(args.key_name), 'key "{}" not existed!'.format(args.key_name)
            acc_config = {}
            if len(args.content):
                for arg in args.content:
                    key, value = arg.split(':')
                    acc_config[key] = value
            else:
                print('[warning] no config contetn specified')
            acc_helper.save_account(args.acc_tag, acc_config, args.key_name, force=args.force)

        elif args.action == 'list':
            accounts = acc_helper.get_accounts()
            print('total accounts: {}'.format(len(accounts)))
            for idx, account in enumerate(accounts, 1):
                print('{}. {}\n{}'.format(idx, account.acc_tag, json.dumps(account.acc_config, indent=2)))
                if account.key_name is not None:
                    print('(encrypted with key "{}")'.format(account.key_name))

        elif args.action == 'delete':
            assert None not in [args.acc_tag], '-a is required'
            acc_helper.delete_account(args.acc_tag)

    def _show_key(self, key_name):
        from .helpers import KeyHelper

        if not KeyHelper.key_exists(key_name):
            print('[error] key "{}" not exists'.format(key_name))

        else:
            _, public_key_path = KeyHelper.get_key_paths(key_name)
            print('showing "{}" keys:'.format(key_name))
            print()
            print('1. private_key (path: (hidden)):')
            print()
            print('(hidden)')
            print()
            with open(public_key_path) as f:
                print('2. public_key (path: {}):'.format(public_key_path))
                print()
                print(''.join(f.readlines()))
                print()
            print('to save the public key in another location')
            print('please create a file with name "{}.pub"'.format(key_name))
            print('then copy/paste the above public key to the file (including the BEGIN/END lines)')

    def _list_keys(self):
        from .helpers import KeyHelper
        key_names = KeyHelper.get_existed_key_names()
        print('total keys: {}'.format(len(key_names)))
        print('\n'.join(['{}. {}'.format(idx, name) for idx, name in enumerate(key_names, 1)]))

    def key(self):
        import linker.rsa_encrypter as rsa
        from .helpers import KeyHelper

        parser = argparse.ArgumentParser(description='gun key')
        parser.add_argument('action', choices=['list', 'show', 'new', 'delete'], help='action to take')
        parser.add_argument('-k', '--key_name', default='default_key', help='key naweme')
        parser.add_argument('-f', '--force', action='store_true', help='force delete/save')
        args = parser.parse_args(sys.argv[2:])

        if args.action == 'new':
            if KeyHelper.key_exists(args.key_name) and not args.force:
                print('[warning] skip adding "{}", key already existed'.format(args.key_name))
                print('to overwrite, please specify -f')
            else:
                private_key_path, public_key_path = KeyHelper.get_key_paths(args.key_name)
                private_key, public_key = rsa.gen_rsa_keys()
                KeyHelper.write_key_to_file(private_key, private_key_path)
                KeyHelper.write_key_to_file(public_key, public_key_path)
                self._show_key(args.key_name)

        elif args.action == 'show':
            self._show_key(args.key_name)

        elif args.action == 'delete':
            if not KeyHelper.key_exists(args.key_name):
                print('[warning] skipped, key "{}" not exists'.format(args.key_name))

            else:
                private_key_path, public_key_path = KeyHelper.get_key_paths(args.key_name)
                os.remove(private_key_path)
                os.remove(public_key_path)
                print('[info] key "{}" removed'.format(args.key_name))

        elif args.action == 'list':
            self._list_keys()

    def db(self):
        from .database.models import Base
        from .database.connector import DatabaseConnector

        parser = argparse.ArgumentParser(description='gun db')
        parser.add_argument('action', choices=['create', 'init', 'drop', 'print'], help='actions to take')
        parser.add_argument('--db', default='db_core', help='db to operate on')
        args = parser.parse_args(sys.argv[2:])

        db_name = args.db
        if args.action == 'create':
            print('creating databse "{}"'.format(db_name))
            engine = DatabaseConnector(db='').get_engine()
            engine.execute('CREATE DATABASE IF NOT EXISTS {}'.format(db_name))
            print('done!')

        elif args.action == 'init':
            # create tables
            print('creating tables on {}...'.format(db_name))
            Base.metadata.create_all(bind=DatabaseConnector(db=db_name).get_engine())
            print('done!')

        elif args.action == 'drop':
            engine = DatabaseConnector(db='').get_engine()
            print('to drop db "{}"...'.format(db_name))
            engine.execute('DROP DATABASE {}'.format(db_name))
            print('done!')

        elif args.action == 'print':
            engine = DatabaseConnector()
            print(engine)

    def encrypt(self):
        from .helpers import KeyHelper
        from linker import rsa_encrypter as rsa

        parser = argparse.ArgumentParser()
        parser.add_argument('-k', '--key_name', default='default_key', help='specify the key to use in encryption')
        parser.add_argument('data', help='the data to encrypt')
        args = parser.parse_args(sys.argv[2:])

        assert KeyHelper.key_exists(args.key_name), 'key "{}" not found'.format(args.key_name)

        _, public_key_path = KeyHelper.get_key_paths(args.key_name)
        encrypted_data = rsa.encode_msg(args.data, public_pem_file=public_key_path)
        print('encrypted data:')
        print()
        print(encrypted_data)
        print()

    def decrypt(self):
        from .helpers import KeyHelper
        from linker import rsa_encrypter as rsa

        parser = argparse.ArgumentParser()
        parser.add_argument('-k', '--key_name', default='default_key', help='specify the key to use in decryption')
        parser.add_argument('data', help='the data to decrypt')
        args = parser.parse_args(sys.argv[2:])

        assert KeyHelper.key_exists(args.key_name), 'key "{}" not found'.format(args.key_name)

        private_key_path, _ = KeyHelper.get_key_paths(args.key_name)
        decrypted_data = rsa.decode_msg(args.data, private_pem_file=private_key_path)
        print('decrypted data:')
        print()
        print(decrypted_data)
        print()
